package de.kif.backend.util

import de.kif.backend.repository.*
import de.kif.backend.route.api.error
import de.kif.backend.route.api.success
import de.kif.common.Message
import de.kif.common.MessageBox
import de.kif.common.MessageType
import de.kif.common.RepositoryType
import io.ktor.application.call
import io.ktor.http.HttpStatusCode
import io.ktor.routing.Route
import io.ktor.routing.get

object PushService {

    var leastValidTimestamp = System.currentTimeMillis()

    /**
     * Save the message with the current timestamp
     */
    fun notify(type: MessageType, repository: RepositoryType, id: Long) {
        val timestamp = System.currentTimeMillis()
        val message = Message(type, repository, id)
    }

    /**
     * Get all messages created after the given timestamp.
     * The return Box has the current timestamp. If the leastValidTimestamp is less then the current timestamp set the
     * valid flag to false and return an empty message list.
     */
    fun getMessages(timestamp: Long?): MessageBox {
        return MessageBox(
            System.currentTimeMillis(),
            true,
            emptyList()
        )
    }

    /**
     * Delete all messages before the given timestamp
     */
    fun gc(timestamp: Long) {
        leastValidTimestamp = timestamp
    }
}

fun Route.pushService() {
    get("/api/updates") {
        try {
            val timestamp = call.request.queryParameters["timestamp"]?.toLongOrNull()

            val messageBox = PushService.getMessages(timestamp)

            call.success(messageBox)
        } catch (_: Exception) {
            call.error(HttpStatusCode.InternalServerError)
        }
    }

    RoomRepository.registerPushService()
    ScheduleRepository.registerPushService()
    TrackRepository.registerPushService()
    UserRepository.registerPushService()
    WorkGroupRepository.registerPushService()
    PostRepository.registerPushService()
}
